(in-package :pila)

(declaim (optimize (speed 3)))
(defclass word ()
  ((name :initarg :name
         :accessor word-name
         :type string)))
    
(defclass native-word (word)
  ((fn-ptr :initarg :fn-ptr
           :accessor fn-ptr
           :type (function ()))))

(defclass user-word (word)
  ((def-array :initarg :def-array
              :accessor user-word-def-array
              :type (vector word *))))

(defclass boolean-word (word)
  ())

(defun make-user-word (name def-array)
  (make-instance 'user-word
                 :name name
                 :def-array def-array))

(defun make-native-word (name impl)
  (make-instance 'native-word
                 :name name
                 :fn-ptr impl))

(defvar *true-value* (make-instance 'boolean-word :name "#t"))
(defvar *false-value* (make-instance 'boolean-word :name "#f"))

(defmethod print-object ((self boolean-word) stream)
  (write-string
    (format nil "~a" (word-name self))
    stream))

(defun native-bool-to-rpn (x)
  (declare (type boolean x))
  (if x *true-value* *false-value*))

(defun rpn-bool-true-p (x)
  (declare (type boolean-word x))
  (eq x *true-value*))

(defmethod rpn-bool-not ((val boolean-word))
  (if (rpn-bool-true-p val)
      *false-value*
      *true-value*))

(defmethod rpn-bool-and ((x boolean-word) (y boolean-word))
  (native-bool-to-rpn
    (and (eq x *true-value*) (eq y *true-value*))))

(defmethod rpn-bool-or ((x boolean-word) (y boolean-word))
  (native-bool-to-rpn
    (or (eq x *true-value*) (eq y *true-value*))))
