(in-package :pila)

(defparameter +parse-num-digits+ "0123456789abcdef")
(declaim (type simple-string +parse-num-digits+))

(defmacro is-number-p (chr)
  `(or
     (char= ,chr #\0)
     (char= ,chr #\1)
     (char= ,chr #\2)
     (char= ,chr #\3)
     (char= ,chr #\4)
     (char= ,chr #\5)
     (char= ,chr #\6)
     (char= ,chr #\7)
     (char= ,chr #\8)
     (char= ,chr #\9)))

(defun parse-num-base-prefix (num)
  "Detects the base of a number according to its prefix
  Returns two values, the base and the rest of the number"
  (declare (type string num))
  (cond
    ((and (> (length num) 2) (char= (char num 0) #\0))
     (let ((base-specifier-char (char num 1)))
       (case base-specifier-char
         (#\x (values 16 (subseq num 2)))
         (#\o (values 8 (subseq num 2)))
         (otherwise
           (error "Invalid base prefix `0~a`" base-specifier-char)))))
    (t
     (unless (is-number-p (char num 0))
       (error "Not a digit"))
     (values 10 num))))
      
(defun parse-num-base (num &key (base 10))
  "Parses a number from a string, with the specified radix (:base).
  If no base is specified, then it defaults to 10.
  Maximum supported base is 16 (hexadecimal)."
  (declare (type string num)
           (type fixnum base))
  (when (or (zerop base) (> base 16))
    (error "Invalid base ~d" base))
  (when (and (> (length num) 1) (char= (char num 0) #\0))
    (error "Number can't start with zero"))
  (let ((number-value 0))
    (declare (type fixnum number-value))
    (loop for c across num do
      (let ((pos (position c +parse-num-digits+)))
        (when (or (null pos) (> pos base))
            (error "invalid character `~a`" c))
        (setf number-value (+ (* number-value base) pos))))
    number-value))
        
(defun parse-num (num)
  (declare (type string num))
  (when (zerop (length num))
    (error "can't convert an empty string"))
  (multiple-value-bind (base digits) (parse-num-base-prefix num)
      (parse-num-base digits :base base)))
